@@ -12,9 +12,8 @@ def is_up_to_date(a, b):
 
 
 def ext_is_flat(ext):
-    fh = open(ext)
-    t = fh.read(len(MAGIC))
-    fh.close()
+    with open(ext) as fh:
+        t = fh.read(len(MAGIC))
     return MAGIC == t
 
 class FastaRecord(object):
@@ -44,9 +43,8 @@ class FastaRecord(object):
         """
         f = fasta_obj.fasta_name
         if klass.is_current(f):
-            fh = open(f + klass.idx, 'rb')
-            idx = cPickle.load(fh)
-            fh.close()
+            with open(f + klass.idx, 'rb') as fh:
+                idx = cPickle.load(fh)
             if flatten_inplace or ext_is_flat(f + klass.ext): flat = klass.modify_flat(f)
             else: flat = klass.modify_flat(f + klass.ext)
             if flatten_inplace and not ext_is_flat(f + klass.ext):
@@ -55,29 +53,26 @@ class FastaRecord(object):
                 return idx, flat
 
         idx = {}
-        flatfh = open(f + klass.ext, 'wb')
-        for i, (seqid, seq) in enumerate(seqinfo_generator):
-            if flatten_inplace:
-                if i == 0:
-                    flatfh.write('>%s\n' % seqid)
-                else:
-                    flatfh.write('\n>%s\n' % seqid)
-            start = flatfh.tell()
-            flatfh.write(seq)
-            stop = flatfh.tell() 
-            idx[seqid] = (start, stop)
-        flatfh.close()
-            
+        with open(f + klass.ext, 'w') as flatfh:
+            for i, (seqid, seq) in enumerate(seqinfo_generator):
+                if flatten_inplace:
+                    if i == 0:
+                        flatfh.write('>%s\n' % seqid)
+                    else:
+                        flatfh.write('\n>%s\n' % seqid)
+                start = flatfh.tell()
+                flatfh.write(seq)
+                stop = flatfh.tell()
+                idx[seqid] = (start, stop)
+
         if flatten_inplace:
             klass.copy_inplace(flatfh.name, f)
-            fh = open(f + klass.idx, 'wb')
-            cPickle.dump(idx, fh, -1)
-            fh.close()
+            with open(f + klass.idx, 'wb') as fh:
+                cPickle.dump(idx, fh, -1)
             return idx, klass.modify_flat(f)
 
-        fh = open(f + klass.idx, 'wb')
-        cPickle.dump(idx, fh, -1)
-        fh.close()
+        with open(f + klass.idx, 'wb') as fh:
+            cPickle.dump(idx, fh, -1)
         return idx, klass.modify_flat(f + klass.ext)
 
     @classmethod
@@ -90,16 +85,15 @@ class FastaRecord(object):
         os.rename(flat_name, fasta_name)
         # still need the flattend file to show
         # it's current.
-        flatfh = open(fasta_name + klass.ext, 'wb')
-        flatfh.write(MAGIC)
-        flatfh.close()
+        with open(fasta_name + klass.ext, 'w') as flatfh:
+            flatfh.write(MAGIC)
+
 
 
-    
     @classmethod
     def modify_flat(klass, flat_file):
-        return open(flat_file, 'rb')
-    
+        return open(flat_file, 'r')
+
     def _adjust_slice(self, islice):
         l = len(self)
         if not islice.start is None and islice.start < 0:
@@ -208,15 +202,19 @@ class NpyFastaRecord(FastaRecord):
 
     def __getitem__(self, islice):
         d = self.getdata(islice)
-        return d.tostring() if self.as_string else d
+        return d.tostring().decode() if self.as_string else d
 
     @property
     def __array_interface__(self):
+        old_as_string = self.as_string
+        self.as_string = False
+        data = self[:]
+        self.as_string = old_as_string
         return {
             'shape': (len(self), ),
             'typestr': '|S1',
             'version': 3,
-            'data': self[:]
+            'data': data,
         }
 
 
@@ -271,7 +269,7 @@ try:
 
 
             db = HDB(f + klass.idx, tc.HDBOWRITER | tc.HDBOCREAT)
-            flatfh = open(f + klass.ext, 'wb')
+            flatfh = open(f + klass.ext, 'w')
             for i, (seqid, seq) in enumerate(seqinfo_generator):
                 if flatten_inplace:
                     if i == 0:
